@@ -512,7 +512,7 @@ class MultiColumnCompletionMenuControl(UIControl):
         """
         return max(get_cwidth(c.display_text) for c in complete_state.completions) + 1
 
-    def mouse_handler(self, mouse_event: MouseEvent) -> Optional["NotImplemented"]:
+    def mouse_handler(self, mouse_event: MouseEvent) -> "NotImplementedOrNone":
         """
         Handle scroll and click events.
         """
@@ -597,11 +597,11 @@ class MultiColumnCompletionMenuControl(UIControl):
         #       never be focussed.
 
         @kb.add("left", is_global=True, filter=filter)
-        def _(event: E) -> None:
+        def _left(event: E) -> None:
             move()
 
         @kb.add("right", is_global=True, filter=filter)
-        def _(event: E) -> None:
+        def _right(event: E) -> None:
             move(True)
 
         return kb
