@@ -2,6 +2,7 @@
 Interface for an output.
 """
 from abc import ABCMeta, abstractmethod
+from typing import Optional, TextIO
 
 from prompt_toolkit.data_structures import Size
 from prompt_toolkit.styles import Attrs
@@ -24,6 +25,8 @@ class Output(metaclass=ABCMeta):
     :class:`~prompt_toolkit.output.win32.Win32Output`.
     """
 
+    stdout: Optional[TextIO] = None
+
     @abstractmethod
     def fileno(self) -> int:
         " Return the file descriptor to which we can write for the output. "
@@ -143,6 +146,22 @@ class Output(metaclass=ABCMeta):
         (VT100 only.)
         """
 
+    @property
+    def responds_to_cpr(self) -> bool:
+        """
+        `True` if the `Application` can expect to receive a CPR response after
+        calling `ask_for_cpr` (this will come back through the corresponding
+        `Input`).
+
+        This is used to determine the amount of available rows we have below
+        the cursor position. In the first place, we have this so that the drop
+        down autocompletion menus are sized according to the available space.
+
+        On Windows, we don't need this, there we have
+        `get_rows_below_cursor_position`.
+        """
+        return False
+
     @abstractmethod
     def get_size(self) -> Size:
         " Return the size of the output window. "
